@@ -1,44 +1,20 @@
 /* "net_db.c" network database support
  * Copyright (C) 1995,1996,1997,1998,1999,2000,2001 Free Software Foundation, Inc.
  * 
- * This program is free software; you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2, or (at your option)
- * any later version.
- * 
- * This program is distributed in the hope that it will be useful,
+ * This library is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU Lesser General Public
+ * License as published by the Free Software Foundation; either
+ * version 2.1 of the License, or (at your option) any later version.
+ *
+ * This library is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
- * 
- * You should have received a copy of the GNU General Public License
- * along with this software; see the file COPYING.  If not, write to
- * the Free Software Foundation, Inc., 59 Temple Place, Suite 330,
- * Boston, MA 02111-1307 USA
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+ * Lesser General Public License for more details.
  *
- * As a special exception, the Free Software Foundation gives permission
- * for additional uses of the text contained in its release of GUILE.
- *
- * The exception is that, if you link the GUILE library with other files
- * to produce an executable, this does not by itself cause the
- * resulting executable to be covered by the GNU General Public License.
- * Your use of that executable is in no way restricted on account of
- * linking the GUILE library code into it.
- *
- * This exception does not however invalidate any other reasons why
- * the executable file might be covered by the GNU General Public License.
- *
- * This exception applies only to the code released by the
- * Free Software Foundation under the name GUILE.  If you copy
- * code from other Free Software Foundation releases into a copy of
- * GUILE, as the General Public License permits, the exception does
- * not apply to the code that you add in this way.  To avoid misleading
- * anyone as to the status of such modified files, you must delete
- * this exception notice from them.
- *
- * If you write modifications of your own for GUILE, it is your choice
- * whether to permit this exception to apply to your modifications.
- * If you do not wish that, delete this exception notice.  */
+ * You should have received a copy of the GNU Lesser General Public
+ * License along with this library; if not, write to the Free Software
+ * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
+ */
 
 
 
@@ -49,12 +25,17 @@
  */
 
 
+#if HAVE_CONFIG_H
+#  include <config.h>
+#endif
+
 #include <errno.h>
 
 #include "libguile/_scm.h"
 #include "libguile/feature.h"
 #include "libguile/strings.h"
 #include "libguile/vectors.h"
+#include "libguile/dynwind.h"
 
 #include "libguile/validate.h"
 #include "libguile/net_db.h"
@@ -65,8 +46,7 @@
 
 #include <sys/types.h>
 
-#if defined (HAVE_WINSOCK2_H) \
-  && !(defined (__CYGWIN32__) || defined (__CYGWIN__))
+#ifdef HAVE_WINSOCK2_H
 #include <winsock2.h>
 #else
 #include <sys/socket.h>
@@ -75,7 +55,11 @@
 #include <arpa/inet.h>
 #endif
 
-#if !defined (HAVE_H_ERRNO) && !defined (__MINGW32__)
+#ifdef __MINGW32__
+#include "win32-socket.h"
+#endif
+
+#if !defined (HAVE_H_ERRNO) && !defined (__MINGW32__) && !defined (__CYGWIN__)
 /* h_errno not found in netdb.h, maybe this will help.  */
 extern int h_errno;
 #endif
@@ -150,13 +134,13 @@ SCM_DEFINE (scm_gethost, "gethost", 0, 1, 0,
 	    "@code{system-error} or @code{misc_error} keys.")
 #define FUNC_NAME s_scm_gethost
 {
-  SCM ans = scm_c_make_vector (5, SCM_UNSPECIFIED);
-  SCM *ve = SCM_VELTS (ans);
+  SCM result = scm_c_make_vector (5, SCM_UNSPECIFIED);
   SCM lst = SCM_EOL;
   struct hostent *entry;
   struct in_addr inad;
   char **argv;
   int i = 0;
+
   if (SCM_UNBNDP (host))
     {
 #ifdef HAVE_GETHOSTENT
@@ -175,36 +159,38 @@ SCM_DEFINE (scm_gethost, "gethost", 0, 1, 0,
 	  return SCM_BOOL_F;
 	}
     }
-  else if (SCM_STRINGP (host))
+  else if (scm_is_string (host))
     {
-      SCM_STRING_COERCE_0TERMINATION_X (host);
-      entry = gethostbyname (SCM_STRING_CHARS (host));
+      char *str = scm_to_locale_string (host);
+      entry = gethostbyname (str);
+      free (str);
     }
   else
     {
-      inad.s_addr = htonl (SCM_NUM2ULONG (1, host));
+      inad.s_addr = htonl (scm_to_ulong (host));
       entry = gethostbyaddr ((char *) &inad, sizeof (inad), AF_INET);
     }
+
   if (!entry)
     scm_resolv_error (FUNC_NAME, host);
   
-  ve[0] = scm_mem2string (entry->h_name, strlen (entry->h_name));
-  ve[1] = scm_makfromstrs (-1, entry->h_aliases);
-  ve[2] = SCM_MAKINUM (entry->h_addrtype + 0L);
-  ve[3] = SCM_MAKINUM (entry->h_length + 0L);
+  SCM_VECTOR_SET(result, 0, scm_from_locale_string (entry->h_name));
+  SCM_VECTOR_SET(result, 1, scm_makfromstrs (-1, entry->h_aliases));
+  SCM_VECTOR_SET(result, 2, scm_from_int (entry->h_addrtype));
+  SCM_VECTOR_SET(result, 3, scm_from_int (entry->h_length));
   if (sizeof (struct in_addr) != entry->h_length)
     {
-      ve[4] = SCM_BOOL_F;
-      return ans;
+      SCM_VECTOR_SET(result, 4, SCM_BOOL_F);
+      return result;
     }
   for (argv = entry->h_addr_list; argv[i]; i++);
   while (i--)
     {
       inad = *(struct in_addr *) argv[i];
-      lst = scm_cons (scm_ulong2num (ntohl (inad.s_addr)), lst);
+      lst = scm_cons (scm_from_ulong (ntohl (inad.s_addr)), lst);
     }
-  ve[4] = lst;
-  return ans;
+  SCM_VECTOR_SET(result, 4, lst);
+  return result;
 }
 #undef FUNC_NAME
 
@@ -230,12 +216,10 @@ SCM_DEFINE (scm_getnet, "getnet", 0, 1, 0,
 	    "given.")
 #define FUNC_NAME s_scm_getnet
 {
-  SCM ans;
-  SCM *ve;
+  SCM result = scm_c_make_vector (4, SCM_UNSPECIFIED);
   struct netent *entry;
+  int eno;
 
-  ans = scm_c_make_vector (4, SCM_UNSPECIFIED);
-  ve = SCM_VELTS (ans);
   if (SCM_UNBNDP (net))
     {
       entry = getnetent ();
@@ -247,29 +231,33 @@ SCM_DEFINE (scm_getnet, "getnet", 0, 1, 0,
 	  return SCM_BOOL_F;
 	}
     }
-  else if (SCM_STRINGP (net))
+  else if (scm_is_string (net))
     {
-      SCM_STRING_COERCE_0TERMINATION_X (net);
-      entry = getnetbyname (SCM_STRING_CHARS (net));
+      char *str = scm_to_locale_string (net);
+      entry = getnetbyname (str);
+      eno = errno;
+      free (str);
     }
   else
     {
-      unsigned long netnum;
-      netnum = SCM_NUM2ULONG (1, net);
+      unsigned long netnum = scm_to_ulong (net);
       entry = getnetbyaddr (netnum, AF_INET);
+      eno = errno;
     }
+
   if (!entry)
-    SCM_SYSERROR_MSG ("no such network ~A", scm_list_1 (net), errno);
-  ve[0] = scm_mem2string (entry->n_name, strlen (entry->n_name));
-  ve[1] = scm_makfromstrs (-1, entry->n_aliases);
-  ve[2] = SCM_MAKINUM (entry->n_addrtype + 0L);
-  ve[3] = scm_ulong2num (entry->n_net + 0L);
-  return ans;
+    SCM_SYSERROR_MSG ("no such network ~A", scm_list_1 (net), eno);
+
+  SCM_VECTOR_SET(result, 0, scm_from_locale_string (entry->n_name));
+  SCM_VECTOR_SET(result, 1, scm_makfromstrs (-1, entry->n_aliases));
+  SCM_VECTOR_SET(result, 2, scm_from_int (entry->n_addrtype));
+  SCM_VECTOR_SET(result, 3, scm_from_ulong (entry->n_net));
+  return result;
 }
 #undef FUNC_NAME
 #endif
 
-#ifdef HAVE_GETPROTOENT
+#if defined (HAVE_GETPROTOENT) || defined (__MINGW32__)
 SCM_DEFINE (scm_getproto, "getproto", 0, 1, 0, 
             (SCM protocol),
 	    "@deffnx {Scheme Procedure} getprotobyname name\n"
@@ -280,12 +268,10 @@ SCM_DEFINE (scm_getproto, "getproto", 0, 1, 0,
 	    "@code{getprotoent} (see below) if no arguments are supplied.")
 #define FUNC_NAME s_scm_getproto
 {
-  SCM ans;
-  SCM *ve;
+  SCM result = scm_c_make_vector (3, SCM_UNSPECIFIED);
   struct protoent *entry;
+  int eno;
 
-  ans = scm_c_make_vector (3, SCM_UNSPECIFIED);
-  ve = SCM_VELTS (ans);
   if (SCM_UNBNDP (protocol))
     {
       entry = getprotoent ();
@@ -297,43 +283,44 @@ SCM_DEFINE (scm_getproto, "getproto", 0, 1, 0,
 	  return SCM_BOOL_F;
 	}
     }
-  else if (SCM_STRINGP (protocol))
+  else if (scm_is_string (protocol))
     {
-      SCM_STRING_COERCE_0TERMINATION_X (protocol);
-      entry = getprotobyname (SCM_STRING_CHARS (protocol));
+      char *str = scm_to_locale_string (protocol);
+      entry = getprotobyname (str);
+      eno = errno;
+      free (str);
     }
   else
     {
-      unsigned long protonum;
-      protonum = SCM_NUM2ULONG (1, protocol);
+      unsigned long protonum = scm_to_ulong (protocol);
       entry = getprotobynumber (protonum);
+      eno = errno;
     }
+
   if (!entry)
-    SCM_SYSERROR_MSG ("no such protocol ~A", scm_list_1 (protocol), errno);
-  ve[0] = scm_mem2string (entry->p_name, strlen (entry->p_name));
-  ve[1] = scm_makfromstrs (-1, entry->p_aliases);
-  ve[2] = SCM_MAKINUM (entry->p_proto + 0L);
-  return ans;
+    SCM_SYSERROR_MSG ("no such protocol ~A", scm_list_1 (protocol), eno);
+
+  SCM_VECTOR_SET(result, 0, scm_from_locale_string (entry->p_name));
+  SCM_VECTOR_SET(result, 1, scm_makfromstrs (-1, entry->p_aliases));
+  SCM_VECTOR_SET(result, 2, scm_from_int (entry->p_proto));
+  return result;
 }
 #undef FUNC_NAME
 #endif
 
+#if defined (HAVE_GETSERVENT) || defined (__MINGW32__)
 static SCM
 scm_return_entry (struct servent *entry)
 {
-  SCM ans;
-  SCM *ve;
+  SCM result = scm_c_make_vector (4, SCM_UNSPECIFIED);
 
-  ans = scm_c_make_vector (4, SCM_UNSPECIFIED);
-  ve = SCM_VELTS (ans);
-  ve[0] = scm_mem2string (entry->s_name, strlen (entry->s_name));
-  ve[1] = scm_makfromstrs (-1, entry->s_aliases);
-  ve[2] = SCM_MAKINUM (ntohs (entry->s_port) + 0L);
-  ve[3] = scm_mem2string (entry->s_proto, strlen (entry->s_proto));
-  return ans;
+  SCM_VECTOR_SET(result, 0, scm_from_locale_string (entry->s_name));
+  SCM_VECTOR_SET(result, 1, scm_makfromstrs (-1, entry->s_aliases));
+  SCM_VECTOR_SET(result, 2, scm_from_uint16 (ntohs (entry->s_port)));
+  SCM_VECTOR_SET(result, 3, scm_from_locale_string (entry->s_proto));
+  return result;
 }
 
-#ifdef HAVE_GETSERVENT
 SCM_DEFINE (scm_getserv, "getserv", 0, 2, 0,
             (SCM name, SCM protocol),
 	    "@deffnx {Scheme Procedure} getservbyname name protocol\n"
@@ -348,6 +335,9 @@ SCM_DEFINE (scm_getserv, "getserv", 0, 2, 0,
 #define FUNC_NAME s_scm_getserv
 {
   struct servent *entry;
+  char *protoname;
+  int eno;
+
   if (SCM_UNBNDP (name))
     {
       entry = getservent ();
@@ -360,20 +350,29 @@ SCM_DEFINE (scm_getserv, "getserv", 0, 2, 0,
 	}
       return scm_return_entry (entry);
     }
-  SCM_VALIDATE_STRING (2, protocol);
-  SCM_STRING_COERCE_0TERMINATION_X (protocol);
-  if (SCM_STRINGP (name))
+
+  scm_frame_begin (0);
+
+  protoname = scm_to_locale_string (protocol);
+  scm_frame_free (protoname);
+
+  if (scm_is_string (name))
     {
-      SCM_STRING_COERCE_0TERMINATION_X (name);
-      entry = getservbyname (SCM_STRING_CHARS (name), SCM_STRING_CHARS (protocol));
+      char *str = scm_to_locale_string (name);
+      entry = getservbyname (str, protoname);
+      eno = errno;
+      free (str);
     }
   else
     {
-      SCM_VALIDATE_INUM (1,name);
-      entry = getservbyport (htons (SCM_INUM (name)), SCM_STRING_CHARS (protocol));
+      entry = getservbyport (htons (scm_to_int (name)), protoname);
+      eno = errno;
     }
+
   if (!entry)
-    SCM_SYSERROR_MSG("no such service ~A", scm_list_1 (name), errno);
+    SCM_SYSERROR_MSG("no such service ~A", scm_list_1 (name), eno);
+
+  scm_frame_end ();
   return scm_return_entry (entry);
 }
 #undef FUNC_NAME
@@ -389,7 +388,7 @@ SCM_DEFINE (scm_sethost, "sethost", 0, 1, 0,
   if (SCM_UNBNDP (stayopen))
     endhostent ();
   else
-    sethostent (!SCM_FALSEP (stayopen));
+    sethostent (scm_is_true (stayopen));
   return SCM_UNSPECIFIED;
 }
 #undef FUNC_NAME
@@ -405,13 +404,13 @@ SCM_DEFINE (scm_setnet, "setnet", 0, 1, 0,
   if (SCM_UNBNDP (stayopen))
     endnetent ();
   else
-    setnetent (!SCM_FALSEP (stayopen));
+    setnetent (scm_is_true (stayopen));
   return SCM_UNSPECIFIED;
 }
 #undef FUNC_NAME
 #endif
 
-#if defined(HAVE_SETPROTOENT) && defined(HAVE_ENDPROTOENT)
+#if defined (HAVE_SETPROTOENT) && defined (HAVE_ENDPROTOENT) || defined (__MINGW32__)
 SCM_DEFINE (scm_setproto, "setproto", 0, 1, 0, 
             (SCM stayopen),
 	    "If @var{stayopen} is omitted, this is equivalent to @code{endprotoent}.\n"
@@ -421,13 +420,13 @@ SCM_DEFINE (scm_setproto, "setproto", 0, 1, 0,
   if (SCM_UNBNDP (stayopen))
     endprotoent ();
   else
-    setprotoent (!SCM_FALSEP (stayopen));
+    setprotoent (scm_is_true (stayopen));
   return SCM_UNSPECIFIED;
 }
 #undef FUNC_NAME
 #endif
 
-#if defined(HAVE_SETSERVENT) && defined(HAVE_ENDSERVENT)
+#if defined (HAVE_SETSERVENT) && defined (HAVE_ENDSERVENT) || defined (__MINGW32__)
 SCM_DEFINE (scm_setserv, "setserv", 0, 1, 0, 
             (SCM stayopen),
 	    "If @var{stayopen} is omitted, this is equivalent to @code{endservent}.\n"
@@ -437,7 +436,7 @@ SCM_DEFINE (scm_setserv, "setserv", 0, 1, 0,
   if (SCM_UNBNDP (stayopen))
     endservent ();
   else
-    setservent (!SCM_FALSEP (stayopen));
+    setservent (scm_is_true (stayopen));
   return SCM_UNSPECIFIED;
 }
 #undef FUNC_NAME
